@@ -79,7 +79,7 @@ c      *********************
 c               init for nuclin, hadrin
 cc       call haddenC
 cc       call chanwnC
-       call cintModels('epics')
+         call cintModels('epics')  
 
 c       ################
 c                user dependent init
@@ -125,6 +125,7 @@ c             init for 1 event
        FirstInt.x = -100000
        FirstInt.y = -100000
        FirstInt.z = -100000
+       pathInB = 0.
        SumDe = 0.
        Move.Abort = 0     !  if this become non zero for an event, 
                      !  the particle is discarded or event is discarded
@@ -232,189 +233,7 @@ c            inquire  current event number created in this run
        integer nev
        nev = Nevrun
        end
-      subroutine eppush(aTrack)
-      implicit none
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
 
-        record /epTrack/ aTrack
-        character*80 msg
-
-        integer i
-       
-        Stack_pos = Stack_pos + 1
-
-        if(Stack_pos .gt. Max_stack_size) then
-           write(msg, *) 'stack overflow; max=',Max_stack_size
-           call cerrorMsg(msg, 1)
-           call cerrorMsg(
-     *     'You may enlarge EPMAX_STACK in ZepMaxdef.h',1)
-           call cerrorMsg(
-     *     'the last 100  stacked tracks are listed on stdout',1)
-c
-           write(*, *)
-     *         ' #  k charge    e            x           z         wz'
-           do i = Stack_pos, Stack_pos - 100, -1
-              write(*, *) i, Stack(i).p.code, 
-     *             Stack(i).p.charge, Stack(i).p.fm.p(4),
-     *             Stack(i).pos.x,
-     *             Stack(i).pos.y,
-     *             Stack(i).w.z
-           enddo
-           stop 9999
-        endif
-        Stack(Stack_pos) = aTrack
-
-        end
-c       ************************
-c                 put 1ptcl in the stack area (for unknown cn)
-        subroutine epputTrack(aTrack)
-       implicit none
-#include "ZepTrack.h"
-#include "Zep3Vec.h"
-#include "Zcnfig.h"
-
-        record /epTrack/ aTrack  ! input. 
-
-c                 find component # for pos (world coord.)
-c              and  set it in aTrack.cn
-        call eppos2cn(0,  aTrack,  aTrack.cn)
-
-c         convert pos/w into  l.c
-        if(aTrack.cn .le. Det.nct ) then
-
-           call epw2l(aTrack.cn, aTrack.pos, aTrack.pos)
-           call epw2ld(aTrack.cn, aTrack.w,  aTrack.w)
-           call eppush(aTrack)
-        else
-           call
-     *     cerrorMsg(' epputTrack: ptcl is not in detector', 0)
-        endif
-      end
-c       ************************
-c         put 1 ptcl in the stack disk (for unknown cn)
-c   
-        subroutine epputTrk2(io, aTrack)
-       implicit none
-#include "ZepTrack.h"
-#include "Zep3Vec.h"
-#include "Zcnfig.h"
-#include "ZepStack.h"
-
-       integer io  !  disk file number to stack the track
-       record /epTrack/ aTrack  ! input. 
-
-c                 find component # for pos (world coord.)
-c              and  set it in aTrack.cn
-        call eppos2cn(0,  aTrack,  aTrack.cn)
-
-c         convert pos/w into  l.c
-        if(aTrack.cn .le. Det.nct ) then
-           call epw2l(aTrack.cn, aTrack.pos, aTrack.pos)
-           call epw2ld(aTrack.cn, aTrack.w,  aTrack.w)
-           write(io) aTrack
-           StackDisk = io
-        else
-           call
-     *     cerrorMsg(' epputTrk2: ptcl is not in detector', 0)
-        endif
-      end
-c     ********************
-      subroutine epnewComp
-      implicit none
-#include "ZepTrackv.h"
-#include "Zcnfig.h"
-c         when a particle moves  to a new component
-c         the following must be done
-      Cn = cTrack.cn
-      MediaNo = Det.Cn2media(Cn)
-
-c       r.l                         cm   / X0           
-      MaxPath =abs( Det.cmp(Cn).MaxPathL/Media(MediaNo).X0)
-c         Det.cmp(Cn).MaxPathL may be < 0; which means
-c        that the value is set by Epics and abs should be used.
-c         already dummy
-      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
-
-
-      end
-
-
-c      ******************* 
-       subroutine epempty
-       implicit none
-c             clear the stack
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-
-           Stack_pos = 0
-      end
-c     ******************************
-      subroutine eppop(aTrack, icon)
-      implicit none
-c             get 1 particle from the stack area
-c              icon=0:  1 ptcl gotten
-c                  -1:  no more ptcl
-#include  "ZepTrackv.h"
-#include  "ZepStack.h"
-
-       
-c
-       record /epTrack/ aTrack
-       integer icon
-       pathInB = 0.
- 10    continue
-c         do 876 until (ok)
-       if(Stack_pos .gt. 0) then
-          icon=0
-c                vectors are in local.c
-          aTrack = Stack(Stack_pos)     
-c                make cosmos ptcl; e may not be informed. g may  not be,
-c                                  if IncGp=0.
-          call ep2cosPtcl(aTrack.p)
-
-          Stack_pos = Stack_pos - 1
-          Cn = aTrack.cn   ! this may be used in epchckE
-c              do business when a new component appear
-          call epnewComp
-          call epchckE(aTrack, icon)
-
-          if(icon .ne. 0) goto 10
-
-       else
-          if(StackDisk .gt. 0) then
-             read(StackDisk, end=100) aTrack
-             call eppush(aTrack)
-             goto 10
-          endif
- 100      continue
-          StackDisk = 0
-          icon=-1
-       endif
- 876    continue
-
-        end
-c       *************************
-        subroutine epgetTrack(n, aTrack, icon)
-        implicit none
-c            returns a specified track in the stack
-c                 
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-#include  "Zcode.h"
-c
-        integer n  ! input. track in n-th stack position
-        record /epTrack/ aTrack  ! output
-        integer icon             ! output. 0--> ok
-                                 !        non 0--> no n-th stack
-        if(n .ge. 1 .and. n .le. Stack_pos) then                   
-           aTrack = Stack(n)
-           icon = 0
-        else
-           icon = 1
-        endif
-       end
-c       #####################################
        subroutine epfl1
        implicit none
 c           follow 1 particle
@@ -568,6 +387,7 @@ c              local. coord..
 c     *************** see if Move.Track crosses the boundary ******
 c              set Cross=t/f, if t, adjust path and set Trunc=t
       call epifCross(icon)
+      if(icon .eq. 2) return  !  ptcl went void by delicate boundary.
       icon = 0
 cc      if(icon .eq. 0) then
 c     
@@ -744,7 +564,8 @@ c          set cosmos condition(TrackBefMove, TargetMassN)
       tokgpm2 = Media(MediaNo).X0g *10.d0 ! r.l --> kg/m^2
 c                 target mass and Z; not used from v7.0
 c         but projectile is set. 
-      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+c      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+      call ep2cosCond
       call cfixModel( cTrack.p )
       call ciniSmpIntL
 c        call csampNEPIntL  before v7.0
@@ -844,7 +665,7 @@ c                      heavy ion and ke/n is < 0.7 GeV
             endif
 
             if(Media(MediaNo).BirksC1 .gt. 0.) then
-               if( abs(cTrack.p.charge) .eq. 1) then
+               if( abs(cTrack.p.charge) .gt. 1) then
 c                   quenching effect
                   call epOrgCorrec(Media(MediaNo), cTrack.p, dedx, cf)
                else
@@ -1039,7 +860,8 @@ c            call epphotoEp(Media(MediaNo).pe,  E, prob, tphot)
 c          set cosmos condition(TrackBefMove, TargetMassN)
 c                 target mass and Z; thi is not used but
 c              projectile is set
-         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+c         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
+         call ep2cosCond
          call cfixModel( cTrack.p )
          call cghPath(E,  tgp)  ! cosmos function
          tgp = tgp * 0.1   !         to g/cm2
@@ -1217,7 +1039,7 @@ c             see if Move.Track crosses a boundary and set Cross = t/f
 c        If t, reset Move.Track.pos just before the boundary.
 c        set exact boundary in Move.boundary.
 c
-      logical kcon, icon
+      integer kcon, icon
 
 
       real*8 el
@@ -1228,26 +1050,31 @@ c          coord is local coord.
 c       icon =1 indicates something wrong. but 
 c       it should have been corrected safely so we don't
 c       care.
-      if(Move.dl .lt. el) then
-         kcon = 0
-         Move.Cross = .false.
-      else
-         kcon = 1
-         Move.Cross = .true.
-         Move.Trunc = .true.
-         Move.Track.pos.x = Move.boundary.x -
-     *             EpsLeng* cTrack.w.x
-         Move.Track.pos.y = Move.boundary.y -
-     *             EpsLeng* cTrack.w.y
-         Move.Track.pos.z = Move.boundary.z -
-     *             EpsLeng* cTrack.w.z
-c         Move.dl = sqrt(
+c       icon =2  : fail to find boundary. due to delicate problem
+      if(icon .ne.  2) then
+         if(Move.dl .lt. el) then
+            kcon = 0
+            Move.Cross = .false.
+         else
+            kcon = 1
+            Move.Cross = .true.
+            Move.Trunc = .true.
+            Move.Track.pos.x = Move.boundary.x -
+     *           EpsLeng* cTrack.w.x
+            Move.Track.pos.y = Move.boundary.y -
+     *           EpsLeng* cTrack.w.y
+            Move.Track.pos.z = Move.boundary.z -
+     *           EpsLeng* cTrack.w.z
+c            Move.dl = sqrt(
 c     *          (Move.Track.pos.x-cTrack.pos.x)**2 +
 c     *          (Move.Track.pos.y-cTrack.pos.y)**2 +
 c     *          (Move.Track.pos.z-cTrack.pos.z)**2 )
-         Move.dl = el - EpsLeng
-         Move.dt = Move.dl/Media(MediaNo).X0
-         Move.dx = Move.dl/Media(MediaNo).gtocm 
+            Move.dl = el - EpsLeng
+            Move.dt = Move.dl/Media(MediaNo).X0
+            Move.dx = Move.dl/Media(MediaNo).gtocm 
+         endif
+      else
+         kcon = 2
       endif
       end
 c
